import os


class Project:

    def __init__(self, root_path: str):
        self.root = root_path


def from_root_file(file_path: str):
    project_file = os.path.realpath(file_path)
    project_root = os.path.dirname(project_file)
    return Project(project_root)



import os
import shutil

import ps

CHARSET = 'UTF-8'


def create_image(file_path: str, size_bytes: int):
    ps.run(
        ('qemu-img', 'create'),
        ('-f', 'qcow2'),
        (file_path, size_bytes),
    )


def empty_file(path: str):
    write(path, str())


def get_mount_uuid(path: str):
    return ps.run_line(
        ('findmnt',),
        ('--noheadings',),
        ('--output', 'UUID'),
        (path,),
    )


def get_path_mount(path: str):
    return ps.run_line(
        ('stat',),
        ('--format', '%m'),
        (path,),
    )


def get_path_uuid(path: str):
    return get_mount_uuid(get_path_mount(path))


def make_directory(path: str):
    os.makedirs(path, exist_ok=True)


def read_file(file_path: str):
    with open(file_path, 'br') as file_object:
        return file_object.read()


def read_file_lines(file_path: str, charset=CHARSET):
    return read_file_text(file_path).split(os.linesep)


def read_file_text(file_path: str, charset=CHARSET):
    return read_file(file_path).decode(charset)


def wipe(path: str):
    try:
        shutil.rmtree(path)
    except NotADirectoryError:
        os.remove(path)
    except FileNotFoundError:
        pass


def write(file_path: str, text: str, charset=CHARSET):
    with open(file_path, 'bw') as file_object:
        file_object.write(text.encode(charset))



import cmd
import ps

cmd.need('grub-mkimage')

COMPRESSION = 'xz'
ENV_BYTES = 1024
ENV_COMMENT = '#'
ENV_HEADER = f'''{ENV_COMMENT} GRUB Environment Block
'''
MODULES = {
    'i386-pc': [
        ('biosdisk',),
        ('ntldr',),
    ]
}


def make_image(image_format: str, image_path: str, modules: list[str],
               memdisk_path: str, pubkey_path: str = None) -> None:
    args = [
        ('grub-mkimage',),
        ('--compress', COMPRESSION),
        ('--format', image_format),
        ('--output', image_path),
        ('--memdisk', memdisk_path),
    ]
    if pubkey_path:
        args.append(('--pubkey', pubkey_path))
    args.extend(modules)
    if modules := MODULES.get(image_format, None):
        args.extend(modules)
    ps.run(*args)



import subprocess

import txt


def get_tuples_args(tuples) -> list[str]:
    args: list[str] = []
    for item in tuples:
        if type(item) is tuple:
            args.extend(item)
        else:
            args.append(item)
    return args


def run(*tuples) -> subprocess.CompletedProcess:
    return subprocess.run(get_tuples_args(tuples), capture_output=False)


def run_line(*tuples, charset: str = txt.CHARSET) -> str:
    lines = run_lines(*get_tuples_args(tuples), charset=charset)
    return lines[0]


def run_lines(*tuples, charset: str = txt.CHARSET) -> list[str]:
    process = subprocess.run(get_tuples_args(tuples), capture_output=True)
    string = process.stdout.decode(charset)
    return string.rstrip().splitlines()



#! /usr/bin/env python3

import os

import fs


if __name__ == '__main__':
    file_path = os.path.realpath(__file__)
    root_path = os.path.dirname(file_path)
    directory_path = os.path.join(root_path, 'tmp')
    file_path = os.path.join(directory_path, 'file')

    fs.wipe(directory_path)
    fs.make_directory(directory_path)
    fs.write(file_path, 'Martine écrit beaucoup.')
    fs.empty_file(file_path)
    fs.write(file_path, 'Martine écrit moins.')



commands: list[str] = []
packages: list[str] = []


def need(command: str) -> None:
    match command:
        case 'debootstrap':
            package = 'debootstrap'
        case 'mksquashfs' | 'unsquashfs':
            package = 'squashfs-tools'
        case _:
            package = None
    if package:
        if package not in packages:
            packages.append(package)



import cmd
import ps

cmd.need('debootstrap')

BOOTSTRAP_ARCHITECTURE = 'amd64'
BOOTSTRAP_VARIANT = 'minbase'


def bootstrap(root_path: str, suite: str, mirror_location: str):
    command = [
        ('debootstrap',),
        ('--arch', BOOTSTRAP_ARCHITECTURE),
        ('--variant', BOOTSTRAP_VARIANT),
        (suite,),
        (root_path,),
        (mirror_location,),
    ]
    completed_process = ps.run(command)
    return completed_process



import logging
import sys


def get_logger(name: str) -> logging.Logger:
    formatter = logging.Formatter(
        "%(name)s: %(asctime)s | %(levelname)s | %(filename)s:%(lineno)s | %(process)d >>> %(message)s"
    )

    # file_handler = logging.FileHandler('log.txt')
    # file_handler.setFormatter(formatter)
    # file_handler.setLevel(logging.INFO)

    out_handler = logging.StreamHandler(stream=sys.stdout)
    out_handler.setFormatter(formatter)
    out_handler.setLevel(logging.INFO)

    logger = logging.getLogger(name)
    # logger.addHandler(file_handler)
    logger.addHandler(out_handler)
    logger.setLevel(logging.INFO)
    return logger



import ps
import rwx.cmd

rwx.cmd.need('mksquashfs')


def mksquashfs(input_root: str, output_file: str):
    ps.run([
        'mksquashfs',
        input_root,
        output_file,
        '-comp', 'zstd',
        '-Xcompression-level', str(18),
    ])
